@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2018 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -38,11 +38,12 @@ sub _announce {
     my $message = shift;
     my %vars = @_;
 
+    print "\n";
     $self->wprint(\*STDOUT, '', '', __x("Pending kernel upgrade!\n\nRunning kernel version:\n  {kversion}\n\nDiagnostics:\n  {message}\n\nRestarting the system to load the new kernel will not be handled automatically, so you should consider rebooting. [Return]\n",
 					 kversion => $vars{KVERSION},
 					 message => $message,
 		   ));
-    <STDIN>;
+    <STDIN> if (-t *STDIN && -t *STDOUT);
 }
 
 
@@ -76,19 +77,51 @@ You should consider rebooting!
 
 EHINT
 
-    <STDIN>;
+    <STDIN> if (-t *STDIN && -t *STDOUT);
+}
+
+
+sub announce_ucode {
+    my $self = shift;
+    my %vars = @_;
+
+    print "\n";
+    $self->wprint(\*STDOUT, '', '', __x("Pending processor microcode upgrade!\n\nDiagnostics:\n  The currently running processor microcode signature is {CURRENT} which is not the expected microcode signature {AVAIL}.\n\nRestarting the system to load the new processor microcode will not be handled automatically, so you should consider rebooting. [Return]\n",
+			 current => $vars{CURRENT},
+			 avail => $vars{AVAIL},
+		   ));
+    <STDIN> if (-t *STDIN && -t *STDOUT);
 }
 
 
 sub notice($$) {
     my $self = shift;
     my $out = shift;
+
+    return unless($self->{verbosity});
+
     my $indent = ' ';
     $indent .= $1 if($out =~ /^(\s+)/);
 
     $self->wprint(\*STDOUT, '', $indent, "$out\n");
 }
 
+sub vspace {
+    my $self = shift;
+
+    return unless($self->{verbosity});
+
+    $self->SUPER::vspace(\*STDOUT);
+}
+
+
+sub command {
+    my $self = shift;
+    my $out = shift;
+
+    print "$out\n";
+}
+
 
 sub _query($$) {
     my $self = shift;
@@ -101,17 +134,14 @@ sub _query($$) {
 	if($self->{stdio_same}) {
 	    my $s = $self->{stdio_same};
 	    if($s eq 'auto') {
-		$s = ($def eq 'Y' ? __('yes') : __('no'));
-	    }
-	    else {
-		$s = __($s);
+		$s = ($def eq 'Y' ? 'yes' : 'no');
 	    }
 
-	    print "$s\n";
+	    print __($s), "\n";
 	    return $s;
 	}
 
-	$i = <STDIN>;
+	$i = <STDIN> if(-t *STDIN && -t *STDOUT);
 	unless(defined($i)) {
 	    $i = 'n';
 	    last;
@@ -151,7 +181,7 @@ sub query_pkgs($$$$$$) {
 
     delete($self->{stdio_same});
 
-    $self->wprint(\*STDOUT, '', '', __($out)."\n");
+    $self->wprint(\*STDOUT, '', '', "$out\n");
     foreach my $rc (sort keys %$pkgs) {
 	my ($or) = grep { $rc =~ /$_/; } keys %$overrides;
 	my $d = (defined($or) ? ($overrides->{$or} ? 'Y' : 'N') : ($def ? 'N' : 'Y'));
